"""
Copyright (c) 2008 Daniel Stutzbach

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

from math import log, sqrt

alpha = 0.01
beta = 1.0 - alpha

num_players = 1

def init(p):
    global num_players
    num_players += 1
    p.simple_score = 200
    p.elo_score = 1000.0
    p.wins = 10 * PERCENT_MEAN / 100.0
    p.loses = 10 * (1.0 - PERCENT_MEAN / 100.0)
    p.percent_score = 100.0*p.wins/(p.wins+p.loses)

    update_score(p)

def EV(p, stats):
    return p.elo_score / sum(p2.elo_score for p2 in stats)

def calc(stats):
    for p in stats:
        prep_score(p)
    for p in stats:
        p.simple_score *= beta

        s = float(p.win)
        p.elo_score += 300.0/len(stats) * (s - EV(p, stats))

        if p.win: p.wins += len(stats)-1
        else: p.loses += 1
        p.percent_score = 100.0*p.wins/(p.wins+p.loses)

    stats[0].simple_score += log(len(stats)) * 1000.0 * alpha

    for p in stats:
        update_score(p)
    
SIMPLE_MEAN = 241
SIMPLE_STD_DEV = 82

ELO_MEAN = 1035
ELO_STD_DEV = 481

PERCENT_MEAN = 36.2201533384
PERCENT_STD_DEV = 12.9621371487

simple_total = SIMPLE_MEAN
simple_sq_total = SIMPLE_MEAN**2
elo_total = ELO_MEAN
elo_sq_total = ELO_MEAN**2
percent_total = PERCENT_MEAN
percent_sq_total = PERCENT_MEAN**2

def prep_score(p):
    global simple_total, simple_sq_total, elo_total, elo_sq_total
    global percent_total, percent_sq_total
    simple_total -= p.simple_score
    simple_sq_total -= p.simple_score**2
    elo_total -= p.elo_score
    elo_sq_total -= p.elo_score**2
    percent_total -= p.percent_score
    percent_sq_total -= p.percent_score**2

def update_score(p):
    global simple_total, simple_sq_total, elo_total, elo_sq_total
    global percent_total, percent_sq_total
    simple_total += p.simple_score
    simple_sq_total += p.simple_score**2
    elo_total += p.elo_score
    elo_sq_total += p.elo_score**2
    percent_total += p.percent_score
    percent_sq_total += p.percent_score**2

    simple_mean = simple_total / num_players
    simple_std_dev = sqrt(simple_sq_total/num_players - simple_mean**2)
    if not simple_std_dev:
        simple_std_dev = 1.0

    elo_mean = elo_total / num_players
    elo_std_dev = sqrt(elo_sq_total/num_players - elo_mean**2)
    if not elo_std_dev:
        elo_std_dev = 1.0

    percent_mean = percent_total / num_players
    percent_std_dev = sqrt(percent_sq_total/num_players - percent_mean**2)
    if not percent_std_dev:
        percent_std_dev = 1.0

    scores = [(p.simple_score - simple_mean) / simple_std_dev,
              (p.elo_score - elo_mean) / elo_std_dev,
              (p.percent_score - percent_mean) / percent_std_dev]

    p.score = sum(scores)/len(scores) * 100.0 + 300.0
