'''
Copyright (c) 2008 David Lambert (Code Penguin)

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
    
    Based on Glicko-2 http://math.bu.edu/people/mg/glicko/glicko2.doc/example.html
'''
GConst = 173.7178
SysConst = 0.06#0.06-1.1513
from math import pi, sqrt, exp, log
def init(p):
    p.rounds = 0
    p.score = 1500
    p.RD = 350
    p.sigma = 0.06

def g(phi):
    return 1/(sqrt(1 + 3*(pow(phi,2))/pow(pi,2)))

def E(mu,muJ,phiJ):
    return 1/(1 + exp(-g(phiJ)*(mu-muJ)))

def glicko(p, opps, winlose):
    mu,phi,sigma = (p.score - 1500)/GConst, p.RD/GConst, p.sigma
    v,sumGE = 0,0
    for i, Z in enumerate(opps):
        muZ,phiZ,sigmaZ,sZ = (Z.score - 1500)/GConst, Z.RD/GConst, Z.sigma, winlose[i]
        v += (g(phiZ)**2)*E(mu,muZ,phiZ)*(1 - E(mu,muZ,phiZ))
        sumGE += g(phiZ)*(sZ - E(mu,muZ,phiZ))
    v = v**-1
    delta = v * sumGE
    
    a = log(sigma**2)
    xi = a
    x = -1
    while True:
        x = xi
        d = (phi**2) + v + exp(x)
        h1 = (-(x-a))/(SysConst**2) - 0.5*exp(x)/d + 0.5*exp(x)*((delta/d)**2)
        h2 = (-1)/(SysConst**2) - 0.5*exp(x)*((phi**2) + v)/(d**2) + 0.5*(delta**2)*exp(x)*((phi**2) + v - exp(x))/(d**3)
        xi = x - h1/h2
        if abs(xi-x) <= 0.000001: break
    p.sigma = exp(xi/2)
    phiS = sqrt(phi**2 + sigma**2)
    phi = 1/(sqrt((1/(phiS**2))+(1/v)))
    
    mu = mu + (phi**2)*sumGE
    p.score = GConst*mu + 1500
    p.RD = GConst*phi
        

def calc(stats):
    for p in stats:
        if p.win: w = p
        p.rounds += 1

    for p in stats:
        if p.win:
            glicko(p,stats[1:], [1 for i in range(len(stats)-1)])
        else:
            glicko(p,[w],(0,))

if __name__ == '__main__':
    
    class Player:
        def __init__(self,score,RD,sigma):
            self.score = score
            self.RD = RD
            self.sigma = sigma
        
    pM = Player(1500,200,0.06)
    p1 = Player(1400,30,0.06)
    p2 = Player(1550,100,0.06)
    p3 = Player(1700,300,0.06)
    players = (pM, p1,p2,p3)
    
    for x in players:
            print x.score, x.RD, x.sigma
    
    glicko(pM,players[1:], (1,0,0))
    print pM.score




