#thanks to rafsoaken for the swell histogram goodies

try:
    import psyco
    psyco.full()
    print 'psyco enabled'
except ImportError:
    pass


import sys
import os
import time

def histogram(data, nbr_intervals, max_chars):
    print ''
    try:
        ceil
    except NameError:
        from math import ceil, floor

    length = len(data)
    data.sort()
    
    mn = data[0]
    mx = data[-1]
    interval = int((mx-mn)/nbr_intervals)
    
    if interval == 0: return #no histogram available
    
    histo = [] 
    for lowerbound in xrange(int(floor(mn)), int(ceil(mx))+1, interval):
        histo.append(len([score for score in data if (lowerbound <= score < lowerbound+interval)]))
          
    scale = float(max_chars)/max(histo)
    print 'Min value:',mn , ', max value:', mx
    print 'Raw histogram values: ', histo
    for i, stars in enumerate(histo):
        print '%5d'%(mn+interval*i),'to %5d'%(mn+interval*(i+1)-1), ':',           
        print int(ceil(stars*scale))* '*'
        pass


def load_entry(name):
    r = __import__(name,globals(),locals())
    return r

class Player:
    def __init__(self,name):
        self.name = name
        self.score = 0.0

def by_score(a,b):
    if a.score > b.score: return 1
    if a.score < b.score: return -1
    return 0
def by_ascore(a,b):
    if a.ascore > b.ascore: return 1
    if a.ascore < b.ascore: return -1
    return 0
    
def by_name(a,b):
    if a.name > b.name: return 1
    if a.name < b.name: return -1
    return 0

class System:
    def __init__(self,name):
        self.name = name
        self.score = 0.0
        self.cscore = 0.0
        self.ascore = 0.0
        self.value = 0.0
        self.total = 0.0
        self.rounds = 0
        self._rounds = 0
        self.players = {}
        self.duration = 0.0
        
    def my_init(self,p): pass
    def my_calc(self,p): pass
        
    def do_raw(self,names):
        stats = []
        for n,p in enumerate(names):
            if p not in self.players:
                pp = self.players[p] = Player(p)
                self.my_init(pp)
            pp = self.players[p]
            pp.win = (n == 0)
            stats.append(pp)
        self.do_stats(stats)
    
    def do_stats(self,stats):
        self._rounds += 1
        skip = 0
        if '-skip' in sys.argv: skip = 100000
        if self._rounds > skip:
            self.do_guess(stats)
        self.my_calc(stats)
        
    def do_guess(self,stats):
        win = stats[0]
        stats = stats[:]; stats.sort(by_name); stats.sort(by_score)
        guess = stats[-1]
        val = len(stats)
        if win == guess:
            self.value += val
        self.total += val
        self.rounds += 1
        self.score = self.value * 1000.0 / self.total
        self.cscore += self.score
        self.ascore = self.cscore / self.rounds
    
def load_data(fname):
    print 'loading',fname,'...'
    data = []
    for line in open(fname,'r'):
        names = line.strip().split('|')
        data.append(names)
    return data
    
def run_data(data,systems,n=-1):
    for s in systems:
        print '\nrunning',s.name,'...'
        total = 0
        ts = time.time()
        for names in data:
            s.do_raw(names)
            total += 1
            if 0 < n <= total: break
            if (total%5000) == 0:
                print '.',
                sys.stdout.flush()
        te = time.time()
        s.duration = te-ts
        if '-histogram' in sys.argv:
            scorelist = [p.score for p in s.players.values()]
            # draw histogram, split player scores in 10 intervals, use 30 stars for largest interval
            histogram(scorelist, 10, 30)
    
    print '\n'
    print 'Results:'
    print ''
    systems.sort(by_ascore)
    systems.reverse()
    for pl,s in enumerate(systems):
        mn,mx = None,None
        for p in s.players.values():
            if mn is None or p.score < mn: mn = p.score
            if mx is None or p.score > mx: mx = p.score
        print "%2d. %-16s %4.2f %4.2f %2d:%02d [%d,%d]"%(pl+1,s.name,round(s.score,2), round(s.ascore,2),s.duration/60,s.duration%60,mn,mx)
    print ''
    print "Rounds: %d"%len(data)

def main():
    systems = []
    n = -1
    if '*.py' in sys.argv:
        import glob
        sys.argv.extend(glob.glob('*.py'))
        sys.argv.remove('*.py')
        
    for v in sys.argv[1:]:
        if v[0] == '-': continue
        try:
            n = int(v)
            print 'limit to %d rounds'%n
            continue
        except:
            pass
        v = v.replace('.py','')
        if v == 'gprize':continue
        fname = '%s.py'%v
        if os.path.exists(fname):
            mod = load_entry(v)
            s = System(v)
            s.my_calc = mod.calc
            try: s.my_init = mod.init
            except: pass
            print 'loaded %s'%fname
            systems.append(s)
        else:
            print "can't find %s"%fname
            
    print ''
    if len(systems):
        fname = 'data.txt'
        data = load_data(fname)
        run_data(data,systems,n)
    else:
        print """python gprize.py [number_of_rounds_to_play] [names_of_other_modules]

for example:
    python gprize.py 10000 my_module
    
    will run the script testing the first 10000 rounds.  It will load
    in your module and run it.  by default it will run as many rounds as
    are in the data.txt file
    
    OR
    
    python gprize.py *.py
    
    will run the script and load all the modules available and give you
    an output showing which does the best job predicting results.

included modules:
    bad - should always have a really low rating by trying to get it wrong
    noop - should be pretty dumb, it doesn't rate anything
    rounds - another simple example that counts how many rounds you played
    simple - a simple example that sort of maybe works
    percent - a simple rankings system - pretty much what I use right now
    elo - a (likely broken) elo rankings system
"""
    

if __name__ == '__main__':
    main()

