# 2008 - glicko1.py - a python implementation of the glicko-1-rating algorithm by florian breisch
# mailto: info@b-lebt.de
# the glicko-algorithm was invented by mark glickman

# IMPORTANT: this implementation is NOT performance-improved

import math

number_of_ratings = 0

C = 12.5 #(12-13)
P = (3.0 * math.log(10.0)**2) / (math.pi**2 * 400.0**2)
Q = math.log(10.0) / 400.0

#print 'glicko runs with C = %4.2f; P = %4.6f; Q = %4.6f' % (C, P, Q)

###############################################################

def glick_new_rd(player):
	global number_of_ratings, C
	return (math.sqrt(player.rd**2 + C * math.log(1.0 + number_of_ratings - player.ratings)))

def glick_f(player):
	return (1.0 / math.sqrt(1.0 + P * player.rd**2))

def glick_e(player, opponent):
	return (1.0 / (1.0 + 10**(-(player.score - opponent.score) * (glick_f(player) / 400.0))))

def glick_k(player, opponent):
	return ((Q * glick_f(player)) / ((1.0 / player.rd**2) + Q**2 * glick_f(player)**2 * glick_e(player, opponent) * (1.0 - glick_e(player, opponent))))

def glick_rd(player, opponent):
	return (1.0 / math.sqrt((1.0 / player.rd**2) + Q**2 * glick_f(player)**2 * glick_e(player, opponent) * (1.0 - glick_e(player, opponent))))

def glick_score(player, opponent):
	return (player.score + glick_k(player, opponent) * (float(player.win) - glick_e(player, opponent)))
###############################################################
###############################################################

def init(player):
	global number_of_ratings
	player.score = 1720.0
	player.rd = 350.0
	player.ratings = number_of_ratings + 1

def calc(stats):
	global number_of_ratings, C
	number_of_ratings += 1
	for player in stats:
		#print 'the rd is moving from %3.3f to %3.3f with a count of %4d missed ratings.' % (player.rd, glick_new_rd(player), number_of_ratings - player.ratings)
		player.rd = glick_new_rd(player)
		player.ratings = number_of_ratings
	for player in stats[1:]:
		winner_score = glick_score(stats[0], player)
		looser_score = glick_score(player, stats[0])
		stats[0].score = winner_score
		player.score = looser_score
		stats[0].rd = glick_rd(stats[0], player)
		player.rd = glick_rd(player, stats[0])