"""
Copyright (c) 2008 Daniel Stutzbach

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

"""

This is a meta-solution.  It runs several of the better algorithm, keeps
a running track of their mean and standard deviation, normalizes them,
then computes an average score.

My "agthorr" solution is given more weight since it's already a
meta-solution that incorporates three different algorithms.

"""

from math import sqrt

import agthorr, mewotwo, bucanrabi, aikipy, azbuky

class Player(object):
    pass

subscores = ['agthorr', 'mewotwo', 'bucanrabi', 'aikipy', 'azbuky']
weights = [3.0] + ([1.0] * (len(subscores)-1))
modules = {}
for sub in subscores:
    modules[sub] = globals()[sub]

totals = {}
sq_totals = {}
num_players = 0

for sub in subscores:
    totals[sub] = 0
    sq_totals[sub] = 0

def update_score(p):
    scores = []
    for sub in subscores:
        s = getattr(p, sub).score
        totals[sub] += s
        sq_totals[sub] += s**2
        
        m = totals[sub] / num_players
        sd = sqrt(sq_totals[sub]/num_players - m**2)
        if not sd:
            sd = 1.0
        scores.append((s - m) / sd)

    p.score = (sum(scores[i]*weights[i] for i in range(len(weights)))
               /sum(weights) * 100.0 + 300.0)

def init(p):
    global num_players
    num_players += 1
    for sub in subscores:
        setattr(p, sub, Player())
        getattr(p, sub).score = p.score
        getattr(p, sub).name = p.name
        if hasattr(modules[sub], 'init'):
            modules[sub].init(getattr(p, sub))
    update_score(p)

def calc(stats):
    for p in stats:
        for sub in subscores:
            s = getattr(p, sub).score
            totals[sub] -= s
            sq_totals[sub] -= s**2
            getattr(p, sub).win = p.win
    
    for sub in subscores:
        modules[sub].calc([getattr(stat, sub) for stat in stats])
    for p in stats:
        update_score(p)
    
