#Copyright (c) 2008 Martin O'Leary
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.

import math

#The range we want scores to lie in
minscore = 0
maxscore = 100

#A measure of the 'width' of the score distribution. This value seems to work, but feel free to play.
sds = 2.5

#Some parameters that control the algorithm
params = [0.058, 0.085, 0.246, 0.03, 2.37]

#Some magic numbers that we want to precalculate for speed.
A = 8 * (3 - math.pi) / (3 * math.pi * (math.pi - 4))
B = 4 / math.pi
C = math.sqrt(2 * math.pi)

#And some derived parameters
midscore = .5 * (minscore + maxscore)
scale = (midscore - minscore)/(math.sqrt(params[3])*sds)
offset = midscore


def clamp(x, a, b):
    return min(max(a, x), b)
    
def phi(x):
    x2 = .5 * x ** 2
    if x > 0:
        return .5 * (1 + math.sqrt(1 - math.exp(-x2 * (B + A * x2)/(1 + A * x2))))
    else:
        return .5 * (1 - math.sqrt(1 - math.exp(-x2 * (B + A * x2)/(1 + A * x2))))

def n(x):
    return math.exp(-.5 * x ** 2)/C

def v(t):
    if t < -5:
        return -t
    if t > 5:
        return 0
    return n(t)/phi(t)

def w(t):
    val = v(t)
    return val * (val + t)
        
def init(p):
    p.mu = 0
    p.sigma2 = params[3]
    p.score = clamp(offset + scale * (p.mu - params[4] * math.sqrt(p.sigma2)), minscore, maxscore)
    
def calc(stats):
    k = params[4]
    beta2 = params[len(stats) - 2]
    w = stats[0]
    wmu, wsigma2 = w.mu, w.sigma2
    for p in stats[1:]:
        pmu, psigma2 = p.mu, p.sigma2
        c = math.sqrt(2 * beta2 + wsigma2 + psigma2)
        diff = (wmu - pmu)/c
        val = v(diff)/c
        wal = val * (val + diff/c)
        w.mu += wsigma2 * val
        p.mu -= psigma2 * val
        w.sigma2 *= (1 - wsigma2 * wal)
        p.sigma2 *= (1 - psigma2 * wal)
    
    for p in stats:
        p.score = clamp(offset + scale * (p.mu - k * math.sqrt(p.sigma2)), minscore, maxscore)      
        