"""
Copyright (c) 2008 Mihai Maruseac (mihai.maruseac@gmail.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

"""
This script is nearly the same as the percent one but it also gives players
various bonuses to encourage them to play more and to keep their winning streak
higher.
"""

import math

start_score = 100
C = 1.0

def init(p):
    """
    Called when a player is created
    """
    p.score = start_score # default score
    p.wins = 0
    p.loses = 0
    p.streak = 0

def sum_score(stats):
    sum = 0
    for p in stats:
        sum += p.score
    return sum

def calc(stats):
    total_score = sum_score(stats)
    players = len(stats)
    for p in stats:
        if p.win:
            p.wins += players - 1
            if p.streak >= 0:
                p.streak += players - 1
            else:
                p.streak = 0
        else:
            p.loses += 1
            if p.streak <= 0:
                p.streak -= 1
            else:
                p.streak = 0
        fWGF = 100.0 * p.wins / (p.wins + p.loses)
        WLD = p.wins - p.loses
        STRK = p.streak
        SD = (players * p.score - total_score) / (players - 1)
        fWLD = TPF2(WLD)
        fSTRK = TPF3(STRK)
        if p.win:
            fSD = -TPF4(SD)
        else:
            fSD = -TPF5(SD)
        w1 = 10
        w2 = 13
        w3 = 21
        w4 = 14
        p.score = w1 * fWGF + w2 * fWLD + w3 * fSTRK + w4 * fSD

def TPF2(val):
    a = -4
    b = 4
    c = 1.5
    d = 1.5
    if val < a:
        return math.exp((val - a) / d) - 1
    elif val > b:
        return 1 - math.exp((b - val) / c)
    return 0.0

def TPF3(val):
    a = -1
    b = 1
    c = 1.4
    d = 1.4
    if val < a:
        return math.exp((val - a) / d) - 1
    elif val > b:
        return 1 - math.exp((b - val) / c)
    return 0.0

def TPF4(val):
    a = -500
    b = 300
    c = 20
    d = 50
    if val < a:
        return math.exp((val - a) / d) - 1
    elif val > b:
        return 1 - math.exp((b - val) / c)
    return 0.0

def TPF5(val):
    a = -500
    b = 100
    c = 175
    d = 40
    if val < a:
        return math.exp((val - a) / d) - 1
    elif val > b:
        return 1 - math.exp((b - val) / c)
    return 0.0
