"""
The MIT License

Copyright (c) <2008> <copyright uman>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

# General thoughts: this is a modified elo. The score is better than straight elo, and
# the histogram is nicer (the max version-- see note below has an even better histogram)
# Spreading out the ranks is trivial: compute the scores using _score variables, and then
# multiple the _scores by say 10 to have a different display scale.

# Some constants
I = 200           # move the distribution closer to zero start
K = 5             # game points won/lost per round
eK = 6            # adjustment to reward more experienced players
roundsT = 12      # threshold where adjustment kicks in

# "game point" table: player's share of 1 game point, expressed as probailities. Since 
# multiplayer games are harder the loser gets some share of the point total because the
# winner is not always the best player. Now, I'm not sure why (I suspect a python 
# floating-point issue), but for 3P & 4P, the sum of the probabilities being 1.042 
# works better than 1!
gp = { 2:(.8, .2), 3:(.6965, .3455), 4:(.6965, .3455) }

def init(p):
    p.rounds = 0
    p.score = I
    p.K = K

def calc(stats):
    np = len(stats)
    for p in stats:
        if p.win: w,rank = p,p.score

        # BTW, replacing the next two lines with p.K = max(5, K / pow(p.rounds, .5)) when
        # K is set to 55, is almost as good, and the histogram is better, but I thought I'd
        # try this one first.
        if p.rounds > roundsT:
            p.K = eK
        p.rounds += 1

    for p in stats:
        if p.win: continue
        cur = p.score
        p.score += p.K*(gp[np][1] - (1/(1+pow(10,(rank-cur)/600))))
        w.score += w.K*(gp[np][0] - (1/(1+pow(10,(cur-rank)/300))))
